
#ifndef TARTANAFILE_HH
#define TARTANAFILE_HH

#include <map>
#include <vector>
#include <string>

#include "TObject.h"

#include "EAnalyser.hh"
#include "EWNum.hh"

class TH1;

class TArtValIndex
{
public:  
  TArtValIndex(){;}
  TArtValIndex(EAnalyser analyser, EWNum wnum, int index)
    : fAnalyser(analyser), fWNum(wnum), fIndex(index){;}
  virtual ~TArtValIndex(){;}

  bool operator < (const TArtValIndex& right) const
  {
    if(fAnalyser != right.fAnalyser){
      return fAnalyser < right.fAnalyser;
    }else if(fWNum != right.fWNum){
      return fWNum < right.fWNum;
    }else{
      return fIndex < right.fIndex;
    }
  }

  int fAnalyser, fWNum, fIndex;
 
  ClassDef(TArtValIndex,1);
};

class TArtAnaGate
{
public:
  TArtAnaGate()
    : fGate(0), fLow(0), fUp(-1), fInside(true), fAnd(false), fOr(false){;}
  TArtAnaGate(int gate, Double_t low, Double_t up)
    : fGate(gate), fLow(low), fUp(up), fInside(false), fAnd(false), fOr(false){;}
  TArtAnaGate(int gate, bool And, bool Or, const std::vector<int>& gatearray)
    : fGate(gate), fLow(0), fUp(-1), fInside(false), fAnd(And), fOr(Or), fGateArray(gatearray){;}
  virtual ~TArtAnaGate(){;}

  int fGate;
  Double_t fLow, fUp;
  bool fInside;

  bool fAnd, fOr;
  std::vector<int> fGateArray;

  ClassDef(TArtAnaGate,1);
};

class TArtAnaFile
{
public:
  TArtAnaFile(){;}
  virtual ~TArtAnaFile(){;}

  bool Open(const char* filename);
  bool Check();
  void Clear();
  bool Add(EAnalyser analyser, EWNum wnum, int index, Double_t val);
  void Fill();

private:
  bool CalGate(int gate);
  bool IsInside(int gate);
  bool Find(TArtValIndex valindex, Double_t& val);

  bool AddValIndexMap(std::string name, std::vector<TArtValIndex> v_valindex);
  bool AddGateMap(std::string name, int gate);
  bool AddValIndexMapForGate(int gate, std::vector<TArtValIndex> v_valindex);
  bool AddAnaGateMap(int gate, TArtAnaGate anagate);

  std::map<TArtValIndex, Double_t> fValMap;

  std::vector<TH1*> fH1Array;
  std::map<std::string, std::vector<TArtValIndex> > fValIndexMap;
  std::map<std::string, int> fGateMap;

  std::map<int, TArtAnaGate> fAnaGateMap;
  std::map<int, std::vector<TArtValIndex> > fValIndexMapForGate;

  ClassDef(TArtAnaFile,1);
};

#endif
