 
#include <iostream>
#include <vector>
#include <map>
#include <fstream>
#include <sstream>
#include <stdexcept>
#include <algorithm>

#include "TArtAnaFile.hh"

#include "TH1.h"
#include "TH1D.h"
#include "TH2D.h"

#include "TArtAnaClock.hh"

bool TArtAnaFile::Open(const char* filename)
{
  std::ifstream ifs;
  ifs.open(filename, std::ios::in);
  if(!ifs){
    std::cout << "cannot open file" << std::endl;
    return false;
  }

  int countline=0;
  while(1){
    if(ifs.eof()) break;
    std::string str;
    std::getline(ifs, str);
    ++countline;
    if(str.empty()) continue;
    if(str.find("#") != std::string::npos) continue;

    //search command
    std::string command = "";
    {
      std::istringstream iss(str);
      while(1){
	std::string str2;
	std::getline(iss, str2, '<');
	if(iss.eof()) break;
	std::getline(iss, str2, '>');
	if(iss.eof()){
	  std::cout << "anafile error(command sequence is wrong) at line: " << countline << std::endl;	  
	  return false;
	}
	command = str2;
      }
    }

    if(!command.empty()){
      if(command == "and" || command == "or"){
	//search values
	std::vector<double> x;
	{
	  std::istringstream iss(str);
	  double xtemp;
	  std::string str2;
	  std::getline(iss, str2, '>');
	  while(1){
	    std::getline(iss, str2, ',');
	    if(iss.eof()) break;
	    std::istringstream iss2(str2);
	    iss2 >> xtemp;
	    x.push_back(xtemp);
	  }
	  if(x.size() < 2){
	    std::cout << "anafile error(number of \',\' is too small) at line: "
		      << countline << std::endl;
	    return false;
	  }
	}

	{
	  int gate = (int)x[0];
	  std::vector<int> v_andgate;
	  for(int i=1; i<(int)x.size(); ++i){
	    v_andgate.push_back((int)x[i]);
	  }
	  if(command == "and"){
	    if(!AddAnaGateMap(gate, TArtAnaGate(gate, true, false, v_andgate))) return false;
	    if(!AddAnaGateMap(-gate, TArtAnaGate(-gate, true, false, v_andgate))) return false;
	  }else if(command == "or"){
	    if(!AddAnaGateMap(gate, TArtAnaGate(gate, false, true, v_andgate))) return false;
	    if(!AddAnaGateMap(-gate, TArtAnaGate(-gate, false, true, v_andgate))) return false;
	  }
	}
      }else if(command == "gate"){
	//search values
	std::vector<double> x;
	{
	  std::istringstream iss(str);
	  double xtemp;
	  std::string str2;
	  std::getline(iss, str2, '>');
	  while(1){
	    std::getline(iss, str2, ',');
	    if(iss.eof()) break;
	    std::istringstream iss2(str2);
	    iss2 >> xtemp;
	    x.push_back(xtemp);
	  }
	  if(x.size() != 7){
	    std::cout << "anafile error(number of \',\' is wrong) at line: " << countline << std::endl;
	    return false;
	  }
	}

	{
	  int ii=-1;
	  int gate = (int)x[++ii];
	  EAnalyser analyser = (EAnalyser)x[++ii];
	  int indexb = (int)x[++ii];
	  int indexe = (int)x[++ii];
	  EWNum wnum = (EWNum)x[++ii];
	  Double_t low = x[++ii];
	  Double_t up = x[++ii];

	  std::vector<TArtValIndex> v_valindex;
	  for(int loop=0; loop<1+indexe-indexb; ++loop){
	    TArtValIndex x(analyser, wnum, indexb+loop);
	    v_valindex.push_back(x);
	  }
	  if(!AddValIndexMapForGate(gate, v_valindex)) return false;
	  if(!AddValIndexMapForGate(-gate, v_valindex)) return false;
	  if(!AddAnaGateMap(gate, TArtAnaGate(gate, low, up))) return false;
	  if(!AddAnaGateMap(-gate, TArtAnaGate(-gate, low, up))) return false;
	}
      }
    }else{

      //search values
      std::vector<double> x;
      {
	std::istringstream iss(str);
	double xtemp;
	while(1){
	  std::string str2;
	  std::getline(iss, str2, ',');
	  if(iss.eof()) break;
	  std::istringstream iss2(str2);
	  iss2 >> xtemp;
	  x.push_back(xtemp);
	}
	if(x.size() != 7 &&
	   x.size() != 8 &&
	   x.size() !=15){
	  std::cout << "anafile error(number of \',\' is wrong) at line: " << countline << std::endl;
	  return false;
	}
      }

      //search title
      std::string title = "";
      {
	std::istringstream iss(str);
	while(1){
	  std::string str2;
	  std::getline(iss, str2, '"');
	  if(iss.eof()) break;
	  std::getline(iss, str2, '"');
	  if(iss.eof()){
	    std::cout << "anafile error(number of \'\"\' is wrong) at line: " << countline << std::endl;	  
	    return false;
	  }
	  title = str2;
	}
      }

      {
	std::ostringstream oss_name, oss_title;

	static int namecounter = 0;
	oss_name << "h" << namecounter++;

	TH1* hist;
	if(x.size() == 8){//1d hist
	  int ii=-1;
	  int gate = (int)x[++ii];
	  EAnalyser analyserx = (EAnalyser)x[++ii];
	  int indexbx = (int)x[++ii];
	  int indexex = (int)x[++ii];
	  EWNum wnumx = (EWNum)x[++ii];
	  Int_t nbinsx = (int)x[++ii];
	  Double_t xlow = x[++ii];
	  Double_t xup = x[++ii];

	  std::vector<TArtValIndex> v_valindex;
	  for(int loop=0; loop<1+indexex-indexbx; ++loop){
	    TArtValIndex x(analyserx, wnumx, indexbx+loop);
	    v_valindex.push_back(x);
	  }
	  if(!AddValIndexMap(oss_name.str(), v_valindex)) return false;
	  if(!AddGateMap(oss_name.str(), gate)) return false;
	  hist = new TH1D(oss_name.str().c_str(), title.c_str(), nbinsx, xlow, xup);
	}else if(x.size() == 15){//2d hist
	  int ii=-1;
	  int gate = (int)x[++ii];
	  EAnalyser analyserx = (EAnalyser)x[++ii];
	  int indexbx = (int)x[++ii];
	  int indexex = (int)x[++ii];
	  EWNum wnumx = (EWNum)x[++ii];
	  Int_t nbinsx = (int)x[++ii];
	  Double_t xlow = x[++ii];
	  Double_t xup = x[++ii];
      
	  EAnalyser analysery = (EAnalyser)x[++ii];
	  int indexby = (int)x[++ii];
	  int indexey = (int)x[++ii];
	  EWNum wnumy = (EWNum)x[++ii];
	  Int_t nbinsy = (int)x[++ii];
	  Double_t ylow = x[++ii];
	  Double_t yup = x[++ii];

	  if(indexex-indexbx != indexey-indexby){
	    std::cout << "anafile error(number of index loop is wrong) at line: "
		      << countline << std::endl;
	    return false;
	  }
	  std::vector<TArtValIndex> v_valindex;
	  for(int loop=0; loop<1+indexex-indexbx; ++loop){
	    TArtValIndex x(analyserx, wnumx, indexbx+loop);
	    TArtValIndex y(analysery, wnumy, indexby+loop);
	    v_valindex.push_back(x);
	    v_valindex.push_back(y);
	  }
	  if(!AddValIndexMap(oss_name.str(), v_valindex)) return false;
	  if(!AddGateMap(oss_name.str(), gate)) return false;
	  hist = new TH2D(oss_name.str().c_str(), title.c_str(), nbinsx, xlow, xup,
			  nbinsy, ylow, yup);
	}
	fH1Array.push_back(hist);
      }
    }
  }

  return true;
}

bool TArtAnaFile::Check()
{
  bool exist = true;
  for(int i=0; i<(int)fH1Array.size(); ++i){
    TH1* hist = fH1Array[i];
    std::string name = hist->GetName();
    int gate = fGateMap[name];

    if(gate == 0) continue;
    std::map<int, TArtAnaGate>::iterator it = fAnaGateMap.find(gate);
    if(it == fAnaGateMap.end()){
      std::cout << "gate: " << gate << " does not exist." << std::endl;
      exist = false;
    }
  }
  return exist;
}

void TArtAnaFile::Clear()
{
  fValMap.clear();
}

bool TArtAnaFile::Add(EAnalyser analyser, EWNum wnum, int index, Double_t val)
{
  TArtValIndex valindex(analyser, wnum, index);
  std::map<TArtValIndex, Double_t>::iterator it = fValMap.find(valindex);
  if(it == fValMap.end()){
    fValMap.insert(std::pair<TArtValIndex, Double_t>(valindex, val));
    return true;
  }else{
    return false;
  }
}

void TArtAnaFile::Fill()
{
  std::map<int, TArtAnaGate>::iterator it = fAnaGateMap.begin();
  while(it != fAnaGateMap.end()){
    CalGate(it->first);
    ++it;
  }

  //  TArtAnaClock::Start();

  for(int i=0; i<(int)fH1Array.size(); ++i){
    TH1* hist = fH1Array[i];
    std::string name = hist->GetName();
    std::vector<TArtValIndex> v_valindex = fValIndexMap[name];
    int gate = fGateMap[name];
    if(hist->GetDimension() == 1){
      for(int j=0; j<(int)v_valindex.size(); ++j){
	Double_t val;
	if(!Find(v_valindex[j], val)) continue;
	if(IsInside(gate)) hist->Fill(val);
      }
    }else if(hist->GetDimension() == 2){
      for(int j=0; j<(int)v_valindex.size(); j+=2){
	Double_t valx, valy;
	if(!Find(v_valindex[j], valx) || !Find(v_valindex[j+1], valy)) continue;
	if(IsInside(gate)) hist->Fill(valx, valy);
      }
    }
  }

//  TArtAnaClock::Stop();
//  std::cout << "\n time:" << std::endl;
//  TArtAnaClock::Print();
}

bool TArtAnaFile::CalGate(int gate)
{
  if(gate == 0) return true;

  TArtAnaGate& anagate = fAnaGateMap[gate];
  if(anagate.fAnd){
    bool inside = true;
    for(int i=0; i<(int)anagate.fGateArray.size(); ++i){
      int gate = anagate.fGateArray[i];
      inside = inside && CalGate(gate);
    }
    return anagate.fInside = (anagate.fGate > 0) ? inside : !inside;
  }else if(anagate.fOr){
    bool inside = false;
    for(int i=0; i<(int)anagate.fGateArray.size(); ++i){
      int gate = anagate.fGateArray[i];
      inside = inside || CalGate(gate);
    }
    return anagate.fInside = (anagate.fGate > 0) ? inside : !inside;
  }else{
    std::vector<TArtValIndex> v_valindex = fValIndexMapForGate[gate];
    bool inside = false;
    for(int j=0; j<(int)v_valindex.size(); ++j){
      Double_t val;
      if(Find(v_valindex[j], val)){
	if(anagate.fLow < val && anagate.fUp > val){
	  inside = true;
	}
      }
    }
    return anagate.fInside = (anagate.fGate > 0) ? inside : !inside;
  }
}

bool TArtAnaFile::IsInside(int gate)
{
  if(gate == 0) return true;
  else return fAnaGateMap[gate].fInside;
}

bool TArtAnaFile::Find(TArtValIndex valindex, Double_t& val)
{
  std::map<TArtValIndex, Double_t>::iterator it = fValMap.find(valindex);
  if(it == fValMap.end()){
    return false;
  }else{
    val = it->second;
    return true;
  }
}

bool TArtAnaFile::AddValIndexMap(std::string name, std::vector<TArtValIndex> v_valindex)
{
  std::map<std::string, std::vector<TArtValIndex> >::iterator it = fValIndexMap.find(name);
  if(it == fValIndexMap.end()){
    fValIndexMap.insert(std::pair<std::string, std::vector<TArtValIndex> >(name, v_valindex));
    return true;
  }else{
    return false;
  }
}

bool TArtAnaFile::AddGateMap(std::string name, int gate)
{
  std::map<std::string, int>::iterator it = fGateMap.find(name);
  if(it == fGateMap.end()){
    fGateMap.insert(std::pair<std::string, int>(name, gate));
    return true;
  }else{
    return false;
  }
}
 
bool TArtAnaFile::AddValIndexMapForGate(int gate, std::vector<TArtValIndex> v_valindex)
{
  std::map<int, std::vector<TArtValIndex> >::iterator it = fValIndexMapForGate.find(gate);
  if(it == fValIndexMapForGate.end()){
    fValIndexMapForGate.insert(std::pair<int, std::vector<TArtValIndex> >(gate, v_valindex));
    return true;
  }else{
    return false;
  }
}

bool TArtAnaFile::AddAnaGateMap(int gate, TArtAnaGate anagate)
{
  std::map<int, TArtAnaGate>::iterator it = fAnaGateMap.find(gate);
  if(it == fAnaGateMap.end()){
    fAnaGateMap.insert(std::pair<int, TArtAnaGate>(gate, anagate));
    return true;
  }else{
    return false;
  }
}
