#ifndef _TARTRIPSPARA_H_
#define _TARTRIPSPARA_H_

#include <TMatrixD.h>

class TArtRIPSPara : public TNamed {
  
 public:
  TArtRIPSPara();
  TArtRIPSPara(Int_t i, Int_t f, Int_t b, char *mfile, Double_t r);
  ~TArtRIPSPara();

  Int_t GetID() const { return id; }
  Int_t GetForwardFpl() const { return forward_fpl; }
  Int_t GetBackwardFpl() const { return backward_fpl; }
  Double_t GetBrho() const { return brho; }
  TMatrixD * GetMatrix() { return &matrix; }

 private:
  Int_t     id;
  Int_t     forward_fpl;
  Int_t     backward_fpl;
  Double_t  brho;

  // transport 5x6 optical matrix
  // note this matrix is GICOSY format, and 1st order calculation is assumed as of Dec. 2011.
  //
  // (x|x') (x|a') (x|y') (x|b') (x|l')
  // (a|x') (a|a') (a|y') (a|b') (a|l')
  // (y|x') (y|a') (y|y') (y|b') (y|l')
  // (b|x') (b|a') (b|y') (b|b') (b|l')
  // (l|x') (l|a') (l|y') (l|b') (l|l')
  // (d|x') (d|a') (d|y') (d|b') (d|l')
  //
  // l = flight length, d = delta
  // with (without) prime"'" means output (input)

  TMatrixD  matrix;

  ClassDef(TArtRIPSPara,1);

};

#endif
